import type { IPFS } from "ipfs-core";
import { PubSub } from "./pubsub";
import { Store } from "./store";
export interface Peer {
    id: string;
    address: string;
}
export declare class Stack {
    namespace: string;
    ipfs: IPFS;
    id: string;
    store: Store;
    pubsub: PubSub;
    private constructor();
    private get libp2p();
    /**
     * Create stack
     *
     * @param namespace needed to scope different stacks data/events
     * @param ipfs IPFS instance / see `@dstack-js/ipfs` package
     * @returns Stack instance
     */
    static create(namespace: string, ipfs: IPFS): Promise<Stack>;
    /**
     * Get connected peers
     *
     * @returns connected peers list
     */
    peers(): Promise<Peer[]>;
    /**
     * Connect to peer
     *
     * By default IPFS will connect to some peers automatically, no need to use it without a reason
     *
     * @param address MultiAddr to connect
     */
    connect(address: string): Promise<void>;
    /**
     * Listen on new peer connected
     *
     * @param listener Will be called with peer info
     */
    onPeerConnect(listener: (peer: Peer) => void): void;
    /**
     * Start logging debug events
     */
    debug(): void;
}
