"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Stack = void 0;
const tslib_1 = require("tslib");
const pubsub_1 = require("./pubsub");
const store_1 = require("./store");
class Stack {
    constructor(namespace, ipfs, id) {
        this.namespace = namespace;
        this.ipfs = ipfs;
        this.id = id;
        this.pubsub = new pubsub_1.PubSub(ipfs, namespace);
        this.store = new store_1.Store(ipfs, namespace, this.pubsub);
    }
    get libp2p() {
        return this.ipfs.libp2p;
    }
    /**
     * Create stack
     *
     * @param namespace needed to scope different stacks data/events
     * @param ipfs IPFS instance / see `@dstack-js/ipfs` package
     * @returns Stack instance
     */
    static create(namespace, ipfs) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { id } = yield ipfs.id();
            const stack = new Stack(namespace, ipfs, id);
            yield stack.store.start();
            return stack;
        });
    }
    /**
     * Get connected peers
     *
     * @returns connected peers list
     */
    peers() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const peers = yield this.ipfs.swarm.peers();
            return peers.map(({ peer, addr }) => ({
                id: peer,
                address: addr.toString()
            }));
        });
    }
    /**
     * Connect to peer
     *
     * By default IPFS will connect to some peers automatically, no need to use it without a reason
     *
     * @param address MultiAddr to connect
     */
    connect(address) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.ipfs.swarm.connect(address);
        });
    }
    /**
     * Listen on new peer connected
     *
     * @param listener Will be called with peer info
     */
    onPeerConnect(listener) {
        this.libp2p.addressManager.on('peer:connect', (event) => {
            listener({
                id: event.remotePeer.toB58String(),
                address: event.remoteAddr.toString()
            });
        });
    }
    /**
     * Start logging debug events
     */
    debug() {
        this.onPeerConnect((peer) => console.log('New peer connected', peer.id, peer.address));
    }
}
exports.Stack = Stack;
//# sourceMappingURL=stack.js.map